"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const Utils_1 = require("fuse-box/Utils");
const path = require("path");
const realm_utils_1 = require("realm-utils");
const utils_1 = require("./utils");
class NodePluginClass {
    constructor(options) {
        this.options = options;
        if (!options || !options.file) {
            throw new Error("invalid options provided");
        }
        // defaults
        options.moduleFolder = options.moduleFolder || "modules";
        options.identifier = options.identifier || path.basename(options.file, ".node");
        options.relativeDependencies = options.relativeDependencies || [];
        options.root = options.root || undefined;
        // validations
        // ... relative dependencies
        const invalidRelativeDependencies = options.relativeDependencies
            .filter((rd) => !realm_utils_1.utils.isString(rd)
            || !rd.trim()
            || path.isAbsolute(rd));
        if (invalidRelativeDependencies.length > 0) {
            throw new Error("invalid relative dependencies. all entries must be relative path strings.");
        }
        // ... identifier
        if (!/^[a-zA-Z0-9 ._-]+$/.test(options.identifier)) {
            throw new Error("invalid identifier. must be directory-name compatible string: " + options.identifier);
        }
        // ... module folder
        if (!/^[a-zA-Z0-9 ._-]+$/.test(options.moduleFolder)) {
            throw new Error("invalid module folder. must be directory-name compatible string: " + options.moduleFolder);
        }
        this.test = Utils_1.string2RegExp(options.file);
    }
    init(context) {
        context.allowExtension(".node");
    }
    transform(file) {
        const context = file.context;
        const options = this.options;
        if (!file.info.fuseBoxPath) {
            throw new Error("file.info.fuseBoxPath is falsy");
        }
        file.isLoaded = true;
        // directories for dependency structure
        const rootDir = options.root || path.dirname((file.relativePath));
        const rootDirAbs = path.resolve(context.appRoot.toString(), rootDir);
        if (!fs_1.existsSync(rootDirAbs)) {
            throw new Error(`root directory "${rootDirAbs}" doesn't exist`);
        }
        // filename
        const dest = `${options.moduleFolder}/${options.identifier}`;
        const userFile = (!context.hash ? Utils_1.hashString(file.info.fuseBoxPath) + "-" : "") + path.basename(file.info.fuseBoxPath);
        const userDest = path.join(dest, path.dirname(file.info.fuseBoxPath));
        // output file path
        const userPath = path.join(userDest, userFile);
        file.alternativeContent = `module.exports = require("./${Utils_1.joinFuseBoxPath(userDest, userFile)}");`;
        return new Promise((resolve, reject) => {
            fs_1.readFile(file.absPath, (err, data) => {
                if (err) {
                    return reject();
                }
                return context.output
                    .writeToOutputFolder(userPath, data, true)
                    .then((result) => {
                    if (result.hash) {
                        file.alternativeContent = `module.exports = require("./${Utils_1.joinFuseBoxPath(userDest, result.filename)}");`;
                    }
                    if (realm_utils_1.utils.isArray(options.relativeDependencies)) {
                        utils_1.copyTreeSync(rootDirAbs, path.resolve(context.output.dir, dest), options.relativeDependencies);
                    }
                    return resolve();
                })
                    .catch(reject);
            });
        });
    }
}
exports.default = NodePluginClass;
